(function($) {

/**
 * @class Drupal.availabilityCalendar.Command represents a calendar state
 * changing command during the whole creation phase, i.e. from click on a state
 * to the click on the end date.
 */
Drupal.availabilityCalendar.Command = function(cid, fieldContext) {
  this.state = "";
  this.from = null;
  this.to = null;
  this.elt = $(".availability-changes", fieldContext);

  /**
   * Sets the state of the current command and, as this is supposed to be the
   * first parameter to be set, cleans the from and to dates.
   */
  this.setState = function(selectedState) {
    if (selectedState !== undefined) {
      this.state = selectedState;
    }
    this.from = null;
    this.to = null;
    this.show();
  };

  /**
   * Adds a date to the command. If it is the 1st date it will be the from date.
   * If it is the 2nd date it will be the to date, if necessary, swapping the
   * from and to dates.
   *
   * @param Date date
   */
  this.addDate = function(date) {
    if (this.from === null) {
      this.from = date;
    }
    else if (this.to === null) {
      this.to = date;
      if (this.to < this.from) {
        var _from = this.from;
        this.from = this.to;
        this.to = _from;
      }
      if (Drupal.availabilityCalendar.getCalendar(cid).isOvernight()) {
        // Overnight rental: to date = departure date = am only: store as "from"
        // to "to - 1 day". But in the case of clicking twice on the same day,
        // it will be handled as a 1 night booking.
        if (this.to > this.from) {
          this.to.setDate(this.to.getDate() - 1);
        }
      }
    }
    this.show();
  };

  /**
   * @returns Boolean Whether the current command is complete.
   */
  this.isComplete = function() {
    return this.to !== null && this.from !== null && this.state !== '';
  };

  /**
   * Replaces the current command in the accompanying hidden field.
   */
  this.show = function() {
    var val = this.elt.val();
    var pos = val.lastIndexOf("\n") + 1;
    val = val.substr(0, pos) + this.toString();
    this.elt.val(val);
  };

  /**
   * Finishes the current command and starts a new one.
   */
  this.finish = function() {
    this.show();
    this.elt.val(this.elt.val() + "\n");
    this.setState();
  };

  /**
   * @returns String
   *   A string representation of the current command.
   */
  this.toString = function() {
    result = "";
    result += "state: ";
    result += this.state !== "" ? this.state : "-";
    result += " from: ";
    result += this.from !== null ? Drupal.availabilityCalendar.formatIsoDate(this.from) : "-";
    result += " to: ";
    result += this.to !== null ? Drupal.availabilityCalendar.formatIsoDate(this.to) : "-";
    return result;
  };
};

/**
 * @class Drupal.availabilityCalendar.Editor
 *   Provides the glueing code that connects the form elements on entity edit
 *   forms (for entities with an availability calendar field) with the @see
 *   Drupal.availabilityCalendar.Calendar API object and the @see
 *   Drupal.availabilityCalendar.Command class.
 */
Drupal.availabilityCalendar.Editor = function(cid) {
  var calendar = null;
  var fieldContext = null;
  var formRadios = null;
  var command = null;
  var calSelectedDay = null;

  init();

  function init() {
    calendar = Drupal.availabilityCalendar.getCalendar(cid);
    fieldContext = calendar.getCalendar().parents('.form-wrapper').first();
    fieldContext.once('Drupal-availabilityCalendar-Editor', function () {
      formRadios = $(".form-radios.availability-states", fieldContext);
      _initCommand();
      _attach2Checkbox();
      _attach2States();
      _bind2CalendarEvents();
    });
  }

  /**
   * Initialize a new @see Drupal.availabilityCalendar.Command object.
   */
  function _initCommand() {
    command = new Drupal.availabilityCalendar.Command(cid, fieldContext);
    command.setState($(":radio:checked", formRadios).val());
    // Add css_class of states as class to wrapper elements around the radios.
    $(":radio", formRadios).parent().addClass(function() {
      return Drupal.availabilityCalendar.getStates().getClass($(this).children(":radio:").val());
    });
  }

  /**
   * Attach to "enable calendar" checkbox (if it exists).
   */
  function _attach2Checkbox() {
    var enable = $('.availability-enable', fieldContext);
    if (enable.size() > 0 ) {
      $('.availability-details', fieldContext).toggle(enable.filter(':checked').size() > 0);
      enable.click(function () {
        $('.availability-details', fieldContext).toggle('fast');
      });
    }
  }

  /**
   * Attach to state radios events.
   */
  function _attach2States() {
    $("input:radio", formRadios).click(function() {
      // State clicked: remove cal-selected and restart current command.
      removeCalSelected();
      command.setState($(":radio:checked", formRadios).val());
    });
  }

  /**
   * Attach to the calendar calendarclick event.
   */
  function _bind2CalendarEvents() {
    calendar.getCalendar().bind("calendarclick", function(event, date, cid) {
      command.addDate(date);
      if (!command.isComplete()) {
        setCalSelected(command.from);
      }
      else {
        removeCalSelected();
        calendar.changeRangeState(command.from, command.to, command.state);
        command.finish();
      }
    });
  }

  function setCalSelected(day) {
    removeCalSelected();
    calSelectedDay = new Date(day.getTime());
    calendar.addExtraState(calSelectedDay, "cal-selected");
  }

  function removeCalSelected() {
    if (calSelectedDay !== null) {
      calendar.removeExtraState(calSelectedDay, "cal-selected");
    }
    calSelectedDay = null;
  }

  return {
    // Publicly exposed methods:
    init: init
  };
};

/**
 * @property object _editors Collection of calendar editor instances.
 */
Drupal.availabilityCalendar._editors = {};

/**
 * Multiton implementation for accessing calendar editors on the page.
 */
Drupal.availabilityCalendar.getEditor = function(cid) {
  if (Drupal.availabilityCalendar._editors[cid] === undefined) {
    Drupal.availabilityCalendar._editors[cid] = new Drupal.availabilityCalendar.Editor(cid);
  }
  else {
    Drupal.availabilityCalendar._editors[cid].init();
  }
  return Drupal.availabilityCalendar._editors[cid];
};

})(jQuery);
