(function($) {

Drupal.availabilityCalendar = Drupal.availabilityCalendar || {};

/**
 * Helper method to return the top element of an Array.
 */
Drupal.availabilityCalendar.top = function(a)  {
  return a[a.length - 1];
};

/**
 * Helper method to format a date according to the date format defined for user
 * presented dates.
 */
Drupal.availabilityCalendar.formatDate = function(date) {
  return date instanceof Date ? $.datepicker.formatDate(Drupal.settings.availabilityCalendar.displayDateFormat, date) : date;
};

/**
 * Helper method to format a date according to the ISO 8601 date format.
 */
Drupal.availabilityCalendar.formatIsoDate = function(date) {
  return date instanceof Date ? $.datepicker.formatDate('yy-mm-dd', date) : date;
};

/**
 * @class Drupal.availabilityCalendar.States
 *   Represents all availability states. Per state some information is kept:
 *   - Integer sid,
 *   - String cssClass
 *   - Boolean isAvailable
 * @param Array states
 *   Array with information about all possible states.
 * @returns object
 *   States object.
 */
Drupal.availabilityCalendar.States = function(states) {
  var statesById = {};
  var statesByClass = {};

  init(states);

  function init(states) {
    // Add empty state (easiest way to prevent javascript errors).
    states[""] = {sid: 0, css_class: "", is_available: false};
    statesById = {};
    statesByClass = {};
    for (var key in states) {
      if (states.hasOwnProperty(key)) {
        statesById[states[key].sid] = states[key];
        statesByClass[states[key].css_class] = states[key];
      }
    }
  }

  function getAll() {
    return statesById;
  }

  function get(state) {
    if (isNaN(state) && statesByClass[state] !== undefined) {
      return statesByClass[state];
    }
    else if (!isNaN(state) && statesById[state] !== undefined) {
      return statesById[state];
    }
    return {};
  }

  function getClass(state) {
    return get(state)['css_class'] !== undefined ? get(state)['css_class'] : '';
  }

  function isAvailable(state) {
    return get(state)['is_available'] !== undefined ? get(state)['is_available'] : false;
  }

  return {
    init: init,
    getAll: getAll,
    get: get,
    getClass: getClass,
    isAvailable: isAvailable
  };
};

/**
 * Javascript API for Availability Calendar field module.
 *
 * @class Drupal.availabilityCalendar.Calendar
 *   Represents the client-side interface for a given availability calendar.
 *   It is possible to have multiple calendars on the same page.
 *   The cid parameter is used to distinghuish calendars.
 *
 *   This API class features:
 *   - Some methods to change the (visual) status of calendar days. However,
 *     note that it does not update the server-side calendar.
 *   - It defines a 'calendarclick' event and triggers it when the visitor
 *     clicks on a day cell in the calendar. The 'calendarclick' event passes in
 *     a Date object, representing the day that was clicked on, and the cid, to
 *     identify which calendar was clicked on. As only DOM elements can trigger
 *     events, you must bind your custom event handler to the calendar element,
 *     retrieved via AvailabilityCalendar.getCalendar():
 *     @code
 *     myAvailabilityCalendar.getCalendar().bind('calendarclick', function(event, date, cid) {
 *       alert('You clicked on date ' + date + ' of calendar ' + cid);
 *     });
 *     @endcode
 *
 * @constructor
 *   Creates a new AvailabilityCalendar interaction object.
 * @param int|string cid
 *   The calendar id for the calendar we want to interact with.
 *   A string starting with 'new' for not yet existing calendars.
 * @param boolean overnight
 *   Indicates whether this calendar is used for overnight or for day bookings.
 * @param boolean splitDay
 *   Indicates whether this calendar should be visualized using split days.
 * @param string selectable
 *   none|all|available|not-available
 *   Indicates whether this calendar allows interaction by selecting dates,
 *   and if so, what states may be selected
 */
Drupal.availabilityCalendar.Calendar = function(cid, name, overnight, splitDay, selectable) {
  var AM = 1;
  var PM = 2;
  var WHOLE_DAY = 3;
  var _calendarId = "#cal-" + cid;
  var _calendarRange = undefined;
  var _calendarDays = {};
  _init();

  /**
   * Initializes the calendar:
   * - Gives selectable cells the class selectable
   * - Initializes the custom calendarclick event on these cells
   */
  function _init() {
    _initDaysAdministration();
    _initSelectable();
    _initCustomEvents();
  };

  /**
   * Creates an overview of all days, their DOM element and their states.
   */
  function _initDaysAdministration() {
    // Get all calendar months.
    $(".cal-month", getCalendar())
      .filter(function() {
        // Make sure it is a calendar month.
        return $(this).attr("id").split("-", 4).length == 4;
      })
      .each(function() {
        // Get year and month of this calendar month.
        var day = null;
        var idParts = $(this).attr("id").split("-", 4);
        var year = idParts[2];
        var month = idParts[3];
        // Get all day cells of this calendar month.
        $("tbody td", $(this))
          .not(".cal-other, .cal-pastdate")
          .each(function() {
            var cell = $(this);
            // Performance of the expresssion Number(cell.text()) is bad,
            // so use the fact that the set is ordered:
            // http://docs.jquery.com/Release:jQuery_1.3.2#Elements_Returned_in_Document_Order
            day = day === null ? Number(cell.text()) : day + 1;
            var date = Drupal.availabilityCalendar.formatIsoDate(new Date(year, month - 1, day));
            var currentClass = this.className;
            var state = {am: "", pm: ""};
            // Loop through all availability states (not including inherited
            // properties) to extract the availability state from the class.
            // Remove found classes to arrive at the other 'extra' classes.
            for (var key in Drupal.availabilityCalendar.getStates().getAll()) {
              var cssClass = Drupal.availabilityCalendar.getStates().getClass(key);
              if (cssClass !== '') {
                if (cell.hasClass(cssClass)) {
                  // Distribute over am and pm but do not overwrite.
                  state.am = state.am || cssClass;
                  state.pm = state.pm || cssClass;
                  cell.removeClass(cssClass);
                }
                if (cell.hasClass(cssClass + "-am")) {
                  state.am = cssClass;
                  cell.removeClass(cssClass + "-am");
                }
                if (cell.hasClass(cssClass + "-pm")) {
                  state.pm = cssClass;
                  cell.removeClass(cssClass + "-pm");
                }
              }
            }
            state = state.am === state.pm ? state.am : state;
            _calendarDays[date] = {cell: this, states: [state], extraStates: this.className};
            // Restore original class.
            this.className = currentClass;
          });
      });
  };

  /**
   * Makes certain cells selectable.
   */
  function _initSelectable() {
    if (selectable !== "none") {
      for (var day in _calendarDays) {
        if (_calendarDays.hasOwnProperty(day)) {
          switch (selectable) {
            case "all":
              addExtraState(day, "cal-selectable");
              break;
            case "available":
              if (isAvailable(day, AM) || isAvailable(day, PM)) {
                addExtraState(day, "cal-selectable");
              }
              break;
            case "not-available":
              if (!isAvailable(day, AM) || !isAvailable(day, PM)) {
                addExtraState(day, "cal-selectable");
              }
            break;
          }
        }
      }
    }
  };

  /**
   * Initializes the custom events for this calendar.
   *
   * The events are triggered by the calendar element, the div surrounding this
   * calendar. Other javascript thus should bind to that element (retrieved via
   * @see getCalendar()).
   *
   * Currently provided custom events:
   * - calendarclick: comes with a date object and the cid as parameters.
   */
  function _initCustomEvents() {
    getCalendar().click(function(event) {
      // Find out if event originated from a day cell, get the date,
      // and trigger the event on the calendar element.
      var day, month, year;
      var cell = $(event.target).closest("td.cal-selectable");
      if (cell.size() > 0) {
        cell
          .each(function() {
            day = Number($(this).text());
          })
          .closest(".cal-month")
          .filter(function() {
            return $(this).attr("id").split("-", 4).length == 4;
          })
          .each(function() {
            var idParts = $(this).attr("id").split("-", 4);
            year = idParts[2];
            month = idParts[3];
          })
          .closest(_calendarId)
          .triggerHandler("calendarclick", [new Date(year, month - 1, day), cid]);
      }
    });
  };

  /**
   * @returns jQuery
   *   A jQuery object containing the calendar DOM element, that is the div with
   *   id='cal-{cid}' and class='cal' surrounding this calendar. This element
   *   triggers the custom events, thus other javascript should bind its
   *   calendar event handling to the return value of this method.
   */
  function getCalendar() {
    return $(_calendarId);
  };

  /**
   * @returns Integer
   *   The cid of the calendar.
   */
  function getCid() {
    return cid;
  };

  /**
   * @returns String
   *   The name of the calendar.
   */
  function getName() {
    return name;
  };

  /**
   * @returns Boolean
   *   If the calendar is used for overnight or for day rental.
   */
  function isOvernight() {
    return overnight;
  };

  /**
   * @returns Boolean
   *   If the calendar is shown using split days.
   */
  function isSplitDay() {
    return overnight && splitDay;
  };

  /**
   * Returns the date range of the calendar.
   *
   * @returns Object|null
   *   { from: Date, to: Date }
   */
  function getCalendarRange() {
    if (_calendarRange === undefined) {
      var from, to;
      // Get all tables representing calendar months within this calendar,
      // extract the month and update the from/to range.
      $(".cal-month", getCalendar()).each(function() {
        var idParts = $(this).attr("id").split("-", 4);
        if (idParts.length == 4) {
          var year = idParts[2];
          var month = idParts[3];
          var calFrom = new Date(year, month - 1, 1);
          if (from === undefined || from > calFrom) {
            from = calFrom;
          }
          var calTo = new Date(year, month, 0);
          if (to === undefined || to < calTo) {
            to = calTo;
          }
        }
      });
      _calendarRange = from !== undefined && to !== undefined ? {from: from, to: to} : null;
    }
    return _calendarRange;
  };

  /**
   * Returns whether the given date is within the calendar range.
   *
   * @param Date day
   * @returns Boolean
   */
  function isInCalendarRange(day) {
    var range = getCalendarRange();
    return range !== null && range.from <= day && day <= range.to;
  };

  /**
   * Returns the number of months the calendar displays.
   *
   * @returns Integer
   */
  function getNumberOfMonths() {
    var range = getCalendarRange();
    return range === 0
      ? 0
      : 12 * (range.to.getFullYear() - range.from.getFullYear())
        + range.to.getMonth() - range.from.getMonth() + 1;
  };

  /**
   * Returns the state for a given date.
   *
   * @param Date|string day
   * @returns null|Integer|Object
   *   - Integer for a whole day state.
   *   - object for a split day state: { am: int, pm: int }.
   *   - null for a date not within the calendar range.
   */
  function getState(day) {
    var state = null;
    var date = Drupal.availabilityCalendar.formatIsoDate(day);
    if (_calendarDays[date] !== undefined) {
      state = Drupal.availabilityCalendar.top(_calendarDays[date].states);
    }
    return state;
  };

  /**
   * Indicates whether a day state is to be considered available for the given
   * dayPart.
   *
   * @param Integer|String|Object state
   *   A single whole day state (Integer|String) or
   *   A composed am/pm split state (Object): { am: Integer|String, pm: Integer|String }.
   * @param Integer dayPart optional (defaults to whole day)
   *   1 = am
   *   2 = pm
   *   3 = am + pm = whole day
   * @returns Boolean
   *   true if the state is available.
   *   null when the state is not recognised.
   */
  function _isDayStateAvailable(state, dayPart) {
    var available;
    // Ignore dayPart if calendar does not support split days.
    if (dayPart === undefined || !isSplitDay()) {
      dayPart = WHOLE_DAY;
    }
    if (typeof state === "string") {
      available = Drupal.availabilityCalendar.getStates().isAvailable(state);
    }
    else {
      available = true;
      if ((dayPart & AM) !== 0) {
        // am state is to be taken into account.
        available = Drupal.availabilityCalendar.getStates().isAvailable(state.am);
      }
      if ((dayPart & PM) !== 0) {
        // pm state is to be taken into account.
        available = available && Drupal.availabilityCalendar.getStates().isAvailable(state.pm);
      }
    }
    return Boolean(available);
  };

  /**
   * Returns whether the given day is available for the given dayPart.
   *
   * @parma Date|string day
   * @param Integer dayPart optional (defaults to whole day)
   *   1 = am
   *   2 = pm
   *   3 = am + pm = whole day
   * @returns Boolean|null
   *   true if the (part of the) date is available.
   *   false if the (part of the) date is not available.
   *   null if the date is not within the calendar range.
   */
  function isAvailable(day, dayPart) {
    var available = null;
    var state = getState(day);
    if (state !== null) {
      available = _isDayStateAvailable(state, dayPart);
    }
    return available;
  };

  /**
   * Internal function to combine the state and any extra states
   * for the given day to 1 value for the className property.
   *
   * @param string date
   */
  function _setCellClass(date) {
    var state = Drupal.availabilityCalendar.top(_calendarDays[date].states);
    var cssClass = typeof state === "string" ? state : state.am + "-am " + state.pm + "-pm";
    if (_calendarDays[date].extraStates !== '') {
      cssClass += " " + _calendarDays[date].extraStates;
    }
    _calendarDays[date].cell.className = cssClass;
  };

  /**
   * Changes the availability state of the given (part of the) day.
   *
   * @param Date|string day
   * @param Integer|String state
   * @param Integer dayPart optional (defaults to whole day)
   *   1 = am
   *   2 = pm
   *   3 = am + pm = whole day
   */
  function changeState(day, state, dayPart) {
    var date = Drupal.availabilityCalendar.formatIsoDate(day);
    state = Drupal.availabilityCalendar.getStates().getClass(state);
    if (_calendarDays[date] !== undefined) {
      // Ignore dayPart if calendar does not support split days.
      if (dayPart === undefined || !isSplitDay()) {
        dayPart = WHOLE_DAY;
      }

      // Determine new composite state: get a clone of the current state, ...
      var newState = Drupal.availabilityCalendar.top(_calendarDays[date].states);
      if (typeof newState === "string") {
        newState = {am: newState, pm: newState};
      }
      else {
        newState = {am: newState.am, pm: newState.pm};
      }
      // ... overwrite both dayPart's (if they should be changed), ...
      if ((dayPart & AM) !== 0) {
        newState.am = state;
      }
      if ((dayPart & PM) !== 0) {
        newState.pm = state;
      }
      // ... normalize back to a single (string) or double (array) state, ...
      newState = newState.am === newState.pm ? newState.am : newState;
      _calendarDays[date].states.push(newState);
      // ... and set the class on the accompanying cell.
      _setCellClass(date);
    }
  };

  /**
   * Restores the availability state of the given (part of the) day to its
   * previous value.
   *
   * @param Date|string day
   */
  function restoreState(day) {
    var date = Drupal.availabilityCalendar.formatIsoDate(day);
    if (_calendarDays[date] !== undefined) {
      // Remove current state (if not the original state)
      if (_calendarDays[date].states.length > 1) {
        _calendarDays[date].states.pop();
      }
      // And set the class on the accompanying cell
      _setCellClass(date);
    }
  };

  /**
   * Sets an extra state on the given (part of the) day.
   * Extra states do not mix with or replace the availability settings.
   *
   * @param Date|string day
   * @param String extraState
   */
  function addExtraState(day, extraState) {
    var date = Drupal.availabilityCalendar.formatIsoDate(day);
    if (_calendarDays[date] !== undefined) {
      // Only add if extra state is not already set.
      var extraStates = " " + _calendarDays[date].extraStates + " ";
      if (extraStates.indexOf(" " + extraState + " ") < 0) {
        _calendarDays[date].extraStates = jQuery.trim(extraStates + " " + extraState);
        // And set the class on the accompanying cell.
        _setCellClass(date);
      }
    }
  };

  /**
   * Removes an extra state of the given (part of the) day.
   * Extra states do not mix with or replace the availability settings.
   *
   * @param Date|string day
   * @param String state
   */
  function removeExtraState(day, extraState) {
    var date = Drupal.availabilityCalendar.formatIsoDate(day);
    if (_calendarDays[date] !== undefined) {
      // Remove state (if set).
      var extraStates = " " + _calendarDays[date].extraStates + " ";
      extraStates = extraStates.replace(" " + extraState + " ", " ");
      _calendarDays[date].extraStates = jQuery.trim(extraStates);
      // And set the class on the accompanying cell.
      _setCellClass(date);
    }
  };

  /**
   * Returns whether all dates in the given range are available.
   *
   * In the split day situation we check from 'from pm' to '(to+1) am'
   * In the whole day situation we check from 'from' to 'to'.
   *
   * @param Date from
   * @param Date to
   * @returns Boolean|null
   *   true if the whole range is available,
   *   false if not the whole range is available,
   *   null if the given date range is not fully within the calendar range.
   */
  function isRangeAvailable(from, to) {
    var available = null;
    var dayPart = WHOLE_DAY;
    // Create a clone for internal use.
    var _to = new Date(to.getTime());
    if (isSplitDay()) {
      // We have to check for the next morning after the to date as well.
      _to.setDate(_to.getDate() + 1);
      // We start with checking only the afternoon.
      dayPart = PM;
    }
    if (isInCalendarRange(from) && isInCalendarRange(_to)) {
      available = true;
      var day = new Date(from.getTime());
      while (available && day <= _to) {
        available = isAvailable(day, dayPart);
        day.setDate(day.getDate() + 1);
        dayPart = day > to ? AM : WHOLE_DAY;
      }
    }
    return available;
  };

  /**
   * Sets all days in the from - to range to the given state.
   *
   * @param Date from
   * @param Date to
   * @param Integer|String state
   */
  function changeRangeState(from, to, state) {
    loopRange(from, to, changeState, state);
  };

  /**
   * Restores all days in the from - to range to their previous state.
   *
   * @param Date from
   * @param Date to
   */
  function restoreRangeState(from, to) {
    loopRange(from, to, restoreState);
  };

  /**
   * Adds the given extra state to all days in the from - to range.
   *
   * Extra states do not mix with or replace the availability settings.
   *
   * @param Date from
   * @param Date to
   * @param String extraState
   */
  function addRangeExtraState(from, to, extraState) {
    loopRange(from, to, addExtraState, extraState);
  };

  /**
   * Removes the given state from all days in the from - to range.
   *
   * Extra states do not mix with or replace the availability settings.
   *
   * @param Date from
   * @param Date to
   * @param String extraState
   */
  function removeRangeExtraState(from, to, extraState) {
    loopRange(from, to, removeExtraState, extraState);
  };

  /**
   * Lopps through a range of dates, applying a callback function.
   *
   * In the split day situation we loop from 'from pm' to 'to+1 am'.
   * In the whole day situation we loop from 'from' up to and including 'to'.
   *
   * @param Date from
   * @param Date to
   * @param Function callback
   * @param mixed arg
   *   An optional additional argument that may be passed to the callback.
   */
  function loopRange(from, to, callback, arg) {
    var range = getCalendarRange();
    if (range !== null) {
      // Create clones for internal use.
      var _from = new Date(from.getTime());
      var _to = new Date(to.getTime());

      // Adjust dates to fall within the calendar range if necessary.
      if (range.from > _from) {
        _from = new Date(range.from.getTime());
      }
      if (range.to < _to) {
        _to = new Date(range.to.getTime());
      }

      // Set daypart and adjust daypart and _to for split day displaying.
      var dayPart = WHOLE_DAY;
      if (isSplitDay()) {
        // We have to check for the next morning after the to date as well.
        _to.setDate(_to.getDate() + 1);
        // We start with checking only the afternoon.
        dayPart = PM;
      }

      // Loop through range of dates. We always pass dayPart to the callback,
      // though it will not always be used.
      var day = new Date(_from.getTime());
      while (day <= _to) {
        if (arg !== undefined) {
          callback(day, arg, dayPart);
        }
        else {
          callback(day, dayPart);
        }
        day.setDate(day.getDate() + 1);
        // What dayPart to check next: whole day or only the morning?
        dayPart = day >= _to && isSplitDay() ? AM : WHOLE_DAY;
      }
    }
  };

  return {
    // Publicly exposed methods:
    getCalendar: getCalendar,
    getCid: getCid,
    getName: getName,
    isOvernight: isOvernight,
    isSplitDay: isSplitDay,
    getCalendarRange: getCalendarRange,
    isInCalendarRange: isInCalendarRange,
    getNumberOfMonths: getNumberOfMonths,
    getState: getState,
    isAvailable: isAvailable,
    changeState: changeState,
    restoreState: restoreState,
    addExtraState: addExtraState,
    removeExtraState: removeExtraState,
    isRangeAvailable: isRangeAvailable,
    changeRangeState: changeRangeState,
    restoreRangeState: restoreRangeState,
    addRangeExtraState: addRangeExtraState,
    removeRangeExtraState: removeRangeExtraState
  };
};

/**
 * @property object _states Collection of calendar states.
 */
Drupal.availabilityCalendar._states = null;

/**
 * Singleton implementation for accessing calendar states.
 *
 * @param boolean create
 *   Whether to create or reinitialize the states. If not set, it just returns
 *   the current object.
 *
 * @return Drupal.availabilityCalendar.States
 */
Drupal.availabilityCalendar.getStates = function(create) {
  if (create === true) {
    if (Drupal.availabilityCalendar._states === null) {
      Drupal.availabilityCalendar._states = new Drupal.availabilityCalendar.States(Drupal.settings.availabilityCalendar.states);
    }
    else {
      // Reinitialize
      Drupal.availabilityCalendar._states.init(Drupal.settings.availabilityCalendar.states);
    }
  }
  return Drupal.availabilityCalendar._states;
};

/**
 * @property object _calendars Collection of calendar instances.
 */
Drupal.availabilityCalendar._calendars = {};

/**
 * Multiton implementation for accessing calendars on the page.
 */
Drupal.availabilityCalendar.getCalendar = function(cid, name, overnight, splitday, selectable) {
  if (Drupal.availabilityCalendar._calendars[cid] === undefined) {
    // We can onlty create a calendar if the other parameters are passed in,
    // otherwise we are requesting a non-existent calendar.
    if (arguments.length > 1) {
      Drupal.availabilityCalendar._calendars[cid] = new Drupal.availabilityCalendar.Calendar(cid, name, overnight, splitday, selectable);
    }
  }
  return Drupal.availabilityCalendar._calendars[cid] !== undefined ? Drupal.availabilityCalendar._calendars[cid] : null;
};

/**
 * @returns A list of all calendars on the current page indexed by cid.
 */
Drupal.availabilityCalendar.getCalendars = function() {
  return Drupal.availabilityCalendar._calendars;
};

})(jQuery);
