(function($) {

/**
 * @class Drupal.availabilityCalendar.BookingFormlet
 *   Provides the glueing code that connects the reservation formlet with the
 *   availability calendar fields on the page.
 *
 *   This class should be able to operate on its own if no calendar is loaded
 *   on the page, though availability_calendars.js remains needed for some
 *   helper functions.
 *
 * @param String formId
 *   The id of the form(let) instance that this object instance is linked to.
 *   The id has to be prefixed with a '#', so it can serve as a jQuery selector.
 * @param Array[Integer] cids
 *   The list of calendar id's to listen to.
 * @param Integer bookedState
 *   The state id (sid) to visually change the state to after the user has
 *   clicked both an arrival and departure date. This gives the visitor visual
 *   feedback and suggest that the state changes to "optionally booked".
 *
 * @see AvailabilityCalendars API object.
 */
Drupal.availabilityCalendar.BookingFormlet = function(formId, cids, bookedState, singleDay) {
  var cid = null;
  var calendar = null;
  var overnight = false;
  var from = null;
  var to = null;
  var calSelectedDay = null;

  init();

  function init() {
    $(formId).once("Drupal-availabilityCalendar-BookingFormlet", function () {
      // Extract values from form elements if preset by the server.
      cid = $('[name="cid"]', formId).val();
      calendar = Drupal.availabilityCalendar.getCalendar(cid);
      overnight = calendar != null ? calendar.isOvernight() : false;

      // from and to are empty or in ISO format (yyyy-mm-dd).
      var fromIso = parseIsoDate($('[name="from_iso"]', formId).val());
      var toIso = parseIsoDate($('[name="to_iso"]', formId).val());

      // Show form in correct state. If we have preset values we use the addDate
      // event handler to also make changes to the calendar.
      if (cid && fromIso !== null) {
        addDate(null, fromIso, cid);
        if (toIso !== null && !singleDay) {
          if (overnight) {
            toIso.setDate(toIso.getDate() + 1);
          }
          addDate(null, toIso, cid);
        }
      }
      else {
        show();
      }

      // Attach to the calendar click events from all calendars to serve.
      for (var i = 0; i < cids.length; i++) {
        var tmpCalendar = Drupal.availabilityCalendar.getCalendar(cids[i]);
        if (tmpCalendar != null) {
          tmpCalendar.getCalendar().bind("calendarclick", addDate);
        }
      }

      // Attach to the click events of the reset buttons.
      $(".acbf-reset-from", formId).click(resetFrom);
      $(".acbf-reset-both", formId).click(resetBoth);
    });
  }

  function parseIsoDate(date) {
    if (date) {
      date = new Date(parseInt(date.substr(0, 4), 10), parseInt(date.substr(5, 2), 10) - 1, parseInt(date.substr(8, 2), 10));
      if (isNaN(date.getTime())) {
        date = null;
      }
    }
    else {
      date = null;
    }
    return date;
  }

  /**
   * Adds a date to the command.
   *
   * - If it is the first date, it will be the from date.
   * - If it is the 2nd date, it will be the to date, swapping the from and to
   *   dates if needed.
   * - If it is a 3rd date, either the from or to date will be changed,
   *   depending on whether the 3rd date is before the current from or not.
   *
   * @paran Event event
   *   The jQuery.Event object.
   * @param Date date
   *   The clicked date.
   * @param int|string calendarCid
   *   The calendar-id or temporarily assigned calendar-id (for new calendars).
   */
  function addDate(event, date, calendarCid) {
    // Create a clone.
    date = new Date(date.getTime());

    // Does this event came from a different calendar on the page?
    if (cid != calendarCid) {
      // Reset the old calendar, both the settings and visually.
      resetBoth();
      show();
      cid = calendarCid;
      calendar = Drupal.availabilityCalendar.getCalendar(cid);
      overnight = calendar != null ? calendar.isOvernight() : false;
      $('[name="cid"]', formId).val(cid);
      $('[name="calendar_label"]', formId).val(calendar != null ? calendar.getName() : "");
    }

    if (to !== null && calendar != null) {
      // Change of range: reset the currently selected range.
      calendar.restoreRangeState(from, to);
    }

    if (singleDay) {
      // Assign the same date but different instances to both from and to.
      from = date;
      to = new Date(date.getTime());
    }
    else {
      if (from === null) {
        // 1st date: assign to from.
        from = date;
      }
      else {
        // 2nd date. Assign to from or to depending on order.
        if (date >= from) {
          // 2nd date is after the 1st date: assign to to.
          to = date;
        }
        else {
          // 2nd date is before the 1st date: assign to from.
          // If to does not already have a value it gets the from date assigned,
          // otherwise it remains unchanged.
          to = to || from;
          from = date;
        }
      }
    }
    show();
  };

  /**
   * Resets the from date and restores the calendar.
   */
  function resetFrom() {
    from = null;
    if (singleDay) {
      to = null;
    }
    show();
    // Stop propagating the event.
    return false;
  };

  /**
   * Resets both dates and restores the calendar.
   */
  function resetBoth() {
    if (calendar != null && from !== null && to !== null) {
      calendar.restoreRangeState(from, to);
    }
    from = to = null;
    show();
    // Stop propagating the event.
    return false;
  }

  /**
   * @returns Boolean Whether the current form values are valid.
   */
  function isValid() {
    return to !== null && from !== null && cid;
  };

  /**
   * Shows the current values, help texts, and enables the submit button.
   */
  function show() {
    if (from === null) {
      // No dates:
      // - Remove the "selected state" from the calendar.
      removeCalSelected();
      // - Hide reset buttons.
      $(".form-reset", formId).css("display", "none");
      // - Disable arrival date field and set help text in it.
      $('[name="from_display"]', formId)
        .attr("disabled", "disabled")
        .addClass("form-button-disabled")
        .val(Drupal.t("Click on an available date in the calendar"));
      // - Disable and empty departure date field.
      $('[name="to_display"]', formId)
        .val("")
        .attr("disabled", "disabled")
        .addClass("form-button-disabled");
      // - Reset iso dates.
      $('[name="from_iso"]', formId).val("");
      $('[name="to_iso"]', formId).val("");
    }
    else {
      // 1 or 2 dates set:
      // - Set value in arrival date field and remove its disabled attribute.
      $('[name="from_display"]', formId)
        .val(Drupal.availabilityCalendar.formatDate(from))
        .removeAttr("disabled")
        .removeClass("form-button-disabled");
      // Set iso from date.
      $('[name="from_iso"]', formId).val(Drupal.availabilityCalendar.formatIsoDate(from));
      if (to === null) {
        // 1 date only:
        // - Show "clear arrival date" button, hide "Clear both dates" button.
        $(".acbf-reset-from", formId).css("display", "inline-block");
        $(".acbf-reset-both", formId).css("display", "none");
        // - Disable departure date field and set help text in it.
        $('[name="to_display"]', formId)
          .val(Drupal.t("Click on your departure date"))
          .attr("disabled", "disabled")
          .addClass("form-button-disabled");
        // - Reset iso to date.
        $('[name="to_iso"]', formId).val("");
        // - Set this single date to the "selected state".
        setCalSelected(from);
      }
      else {
        // 2 dates set:
        // - Remove the "selected state" from the calendar.
        removeCalSelected();
        // - Set iso to date.
        //
        // In day rental this will be the same as the clicked date (= to).
        // In overnight rental situations people click on the departure date and
        // the iso to date will be set to the day before.
        // But in the case of clicking twice on the same day, it will be handled
        // as a 1 day/night booking, even in the overnight rental situation.
        var toIso = new Date(to.getTime());
        if (overnight) {
          if (to > from) {
            // Adjust isoTo by going back 1 day.
            toIso.setDate(toIso.getDate() - 1);
          }
          else {
            // Clicked twice on the same day: handle as if the 2nd click was on
            // the next day.
            to.setDate(to.getDate() + 1);
          }
        }
        $('[name="to_iso"]', formId).val(Drupal.availabilityCalendar.formatIsoDate(toIso));
        // - Set value in departure date field and remove disabled attribute.
        $('[name="to_display"]', formId)
          .val(Drupal.availabilityCalendar.formatDate(to))
          .removeAttr("disabled")
          .removeClass("form-button-disabled");
        // - Hide "clear arrival date" button, show  "Clear both dates" button.
        $(".acbf-reset-from", formId).css("display", "none");
        $(".acbf-reset-both", formId).css("display", "inline-block");
        // - Visually update the calendar.
        if (calendar != null && isValid()) {
          calendar.changeRangeState(from, toIso, bookedState);
        }
      }
    }
    // Enable or diable the submit button depending on the validity of the form.
    if (isValid()) {
      $(".form-submit", formId).removeAttr("disabled").removeClass("form-button-disabled");
    }
    else {
      $(".form-submit", formId).attr("disabled", "disabled").addClass("form-button-disabled");
    }

  };

  function setCalSelected(day) {
    if (calendar !== null) {
      if (calSelectedDay !== null) {
        removeCalSelected();
      }
      calSelectedDay = new Date(day.getTime());
      calendar.addExtraState(calSelectedDay, "cal-selected");
    }
  }

  function removeCalSelected() {
    if (calSelectedDay !== null && calendar !== null) {
      calendar.removeExtraState(calSelectedDay, "cal-selected");
      calSelectedDay = null;
    }
  }

  return {
    // Publicly exposed methods:
    init: init,
    show: show,
    addDate: addDate,
    resetFrom: resetFrom,
    resetBoth: resetBoth,
    isValid: isValid
  };
};

/**
 * @property object _bookingFormlets Collection of booking formlet instances.
 */
Drupal.availabilityCalendar._bookingFormlets = {};

/**
 * Multiton implementation for accessing booking formlets on the page.
 */
Drupal.availabilityCalendar.getBookingFormlet = function(formId, cids, bookedState) {
  if (Drupal.availabilityCalendar._bookingFormlets[formId] === undefined) {
    if (arguments.length > 1) {
      Drupal.availabilityCalendar._bookingFormlets[formId] = new Drupal.availabilityCalendar.BookingFormlet(formId, cids, bookedState);
    }
  }
  else {
    Drupal.availabilityCalendar._bookingFormlets[formId].init();
  }
  return Drupal.availabilityCalendar._bookingFormlets[formId] !== undefined  ? Drupal.availabilityCalendar._bookingFormlets[formId] : null;
};

})(jQuery);
